/*
 * CharValue.h
 *
 * Created on: Nov 1, 2013
 * * Authors: Vincent Wochnik <v.wochnik@gmail.com>
 *
 * Copyright (c) 2013 Cumulocity GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef CHARVALUE_H
#define CHARVALUE_H

#include "config.h"
#include "Value.h"

/**
 * A basic character value. This value type behaves simmilar to a null
 * value if the ommitted string is empty.
 * 
 * Example:
 * @code
 * // A string
 * CharValue str("Hello World!");
 * 
 * // a null value
 * CharValue null("");
 * 
 * // another null value
 * CharValue null2(NULL);
 * @encode
 */
class CharValue : public Value
{
	public:
		CharValue(const char*, bool = false);
		~CharValue();

		uint8_t valueType() const;
		long integerValue() const;
		double floatValue() const;
		const char * characterValue() const;

		size_t write(AbstractDataSink&) const;
		size_t length() const;
		Value* copy() const;

	protected:
		bool escapeCheck();

	private:
		const char *_str;
		bool _escape, _alloc;
};

#endif
