/*
 * StaticData.h
 *
 * Created on: Nov 1, 2013
 * * Authors: Vincent Wochnik <v.wochnik@gmail.com>
 *
 * Copyright (c) 2013 Cumulocity GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef STATICDATA_H
#define STATICDATA_H

#include "config.h"
#include <stddef.h>
#include "DataGenerator.h"

/*
 * A static data store mechanism streaming data to a sink.
 * 
 * Example:
 * @code
 * // given a concrete data sink
 * AbstractDataSink sink;

 * StaticData data("Hello World!");
 * assert(data.writtenLength() == data.writeTo(sink));
 * @encode
 */
class StaticData : public DataGenerator
{
	public:
		StaticData(void*, size_t, bool = false);
		StaticData(const char*, bool = false);
		~StaticData();
		size_t writeTo(AbstractDataSink&) const;
		size_t writtenLength() const;
		DataGenerator* copy() const;

	private:
		void *_buf;
		size_t _len;
		bool _alloc;
};

#endif
